@@ -1,7 +1,12 @@
-import abc,os,functools
+import abc
+import functools
+import os
 from abc import ABC
+
 from typeguard import check_type
-from .opio import Artifact, OPIO, OPIOSign
+
+from .opio import OPIO, Artifact, OPIOSign, Parameter
+
 
 class OP(ABC):
     """
@@ -13,16 +18,19 @@ class OP(ABC):
     """
     progress_total = 1
     progress_current = 0
+
     def __init__(
             self,
             *args,
-            **kwargs) -> None:
+            **kwargs,
+    ) -> None:
         pass
 
     def __setattr__(self, key, value):
         super().__setattr__(key, value)
         if key in ["progress_total", "progress_current"]:
-            with open(os.environ.get("ARGO_PROGRESS_FILE", "ARGO_PROGRESS_FILE"), "w") as f:
+            with open(os.environ.get("ARGO_PROGRESS_FILE",
+                                     "ARGO_PROGRESS_FILE"), "w") as f:
                 f.write("%s/%s" % (self.progress_current, self.progress_total))
 
     @classmethod
@@ -38,7 +46,7 @@ class OP(ABC):
         """
 
     @abc.abstractmethod
-    def execute (
+    def execute(
             self,
             op_in: OPIO,
     ) -> OPIO:
@@ -57,19 +65,27 @@ class OP(ABC):
 
     @staticmethod
     def _check_signature(
-            opio : OPIO,
-            sign : OPIOSign,
+            opio: OPIO,
+            sign: OPIOSign,
     ) -> None:
-        for ii in sign.keys() :
+        for ii in sign.keys():
             if ii not in opio.keys():
-                raise RuntimeError('key %s required in signature is not provided by the opio' % ii)
-        for ii in opio.keys() :
+                if isinstance(sign[ii], Parameter) and hasattr(sign[ii],
+                                                               "default"):
+                    opio[ii] = sign[ii].default
+                else:
+                    raise RuntimeError('key %s required in signature is '
+                                       'not provided by the opio' % ii)
+        for ii in opio.keys():
             if ii not in sign.keys():
-                raise RuntimeError('key %s in OPIO is not in its signature' % ii)
+                raise RuntimeError(
+                    'key %s in OPIO is not in its signature' % ii)
             io = opio[ii]
             ss = sign[ii]
             if isinstance(ss, Artifact):
                 ss = ss.type
+            if isinstance(ss, Parameter):
+                ss = ss.type
             # skip type checking if the variable is None
             if io is not None:
                 check_type(ii, io, ss)
