@@ -1,14 +1,24 @@
-import os, shutil
+import os
+import shutil
 import uuid
-import jsonpickle
-from typing import Set, List
 from pathlib import Path
-from .opio import BigParameter
-from ..utils import assemble_path_list, convert_dflow_list, copy_file, remove_empty_dir_tag
+from typing import List, Set
 
-def handle_input_artifact(name, sign, slices=None, data_root="/tmp"):
-    art_path = data_root + '/inputs/artifacts/' + name
-    if not os.path.exists(art_path): # for optional artifact
+import jsonpickle
+
+from ..config import config
+from ..utils import (assemble_path_list, convert_dflow_list, copy_file,
+                     remove_empty_dir_tag)
+from .opio import BigParameter, Parameter
+
+
+def handle_input_artifact(name, sign, slices=None, data_root="/tmp",
+                          sub_path=None):
+    if sub_path is None:
+        art_path = '%s/inputs/artifacts/%s' % (data_root, name)
+    else:
+        art_path = '%s/inputs/artifacts/%s/%s' % (data_root, name, sub_path)
+    if not os.path.exists(art_path):  # for optional artifact
         return None
     remove_empty_dir_tag(art_path)
     path_list = assemble_path_list(art_path)
@@ -34,6 +44,7 @@ def handle_input_artifact(name, sign, slices=None, data_root="/tmp"):
     elif sign.type == Set[Path]:
         return set(map(Path, path_list))
 
+
 def handle_input_parameter(name, value, sign, slices=None, data_root="/tmp"):
     if "dflow_list_item" in value:
         dflow_list = []
@@ -47,13 +58,18 @@ def handle_input_parameter(name, value, sign, slices=None, data_root="/tmp"):
                 obj = content["value"]
             else:
                 obj = jsonpickle.loads(content["value"])
-    elif sign == str and slices is None:
-        obj = value
     else:
-        obj = jsonpickle.loads(value)
+        if isinstance(sign, Parameter):
+            sign = sign.type
+        if sign == str and slices is None:
+            obj = value
+        else:
+            obj = jsonpickle.loads(value)
 
     if slices is not None:
-        assert isinstance(obj, list), "Only parameters of type list can be sliced, while %s is not list" % obj
+        assert isinstance(
+            obj, list), "Only parameters of type list can be sliced, while %s"\
+            " is not list" % obj
         if isinstance(slices, list):
             obj = [obj[i] for i in slices]
         else:
@@ -61,6 +77,7 @@ def handle_input_parameter(name, value, sign, slices=None, data_root="/tmp"):
 
     return obj
 
+
 def handle_output_artifact(name, value, sign, slices=None, data_root="/tmp"):
     path_list = []
     if sign.type in [str, Path]:
@@ -70,7 +87,8 @@ def handle_output_artifact(name, value, sign, slices=None, data_root="/tmp"):
         else:
             slices = 0
         if value and os.path.exists(str(value)):
-            path_list.append({"dflow_list_item": copy_results(value, name, data_root), "order": slices})
+            path_list.append({"dflow_list_item": copy_results(
+                value, name, data_root), "order": slices})
         else:
             path_list.append({"dflow_list_item": None, "order": slices})
     elif sign.type in [List[str], List[Path], Set[str], Set[Path]]:
@@ -78,31 +96,42 @@ def handle_output_artifact(name, value, sign, slices=None, data_root="/tmp"):
         if slices is not None:
             if isinstance(slices, int):
                 for path in value:
-                    path_list.append(copy_results_and_return_path_item(path, name, slices, data_root))
+                    path_list.append(copy_results_and_return_path_item(
+                        path, name, slices, data_root))
             else:
                 assert len(slices) == len(value)
                 for path, s in zip(value, slices):
                     if isinstance(path, list):
                         for p in path:
-                            path_list.append(copy_results_and_return_path_item(p, name, s, data_root))
+                            path_list.append(
+                                copy_results_and_return_path_item(p, name, s,
+                                                                  data_root))
                     else:
-                        path_list.append(copy_results_and_return_path_item(path, name, s, data_root))
+                        path_list.append(copy_results_and_return_path_item(
+                            path, name, s, data_root))
         else:
             for s, path in enumerate(value):
-                path_list.append(copy_results_and_return_path_item(path, name, s, data_root))
+                path_list.append(copy_results_and_return_path_item(
+                    path, name, s, data_root))
 
-    with open(data_root + "/outputs/artifacts/%s/.dflow.%s" % (name, uuid.uuid4()), "w") as f:
+    os.makedirs(data_root + "/outputs/artifacts/%s/%s" % (name, config[
+            "catalog_dir_name"]), exist_ok=True)
+    with open(data_root + "/outputs/artifacts/%s/%s/%s" % (name, config[
+            "catalog_dir_name"], uuid.uuid4()), "w") as f:
         f.write(jsonpickle.dumps({"path_list": path_list}))
     handle_empty_dir(data_root + "/outputs/artifacts/%s" % name)
-    if slices is not None:
-        with open(data_root + '/outputs/parameters/dflow_%s_path_list' % name, 'w') as f:
+    if config["save_path_as_parameter"]:
+        with open(data_root + '/outputs/parameters/dflow_%s_path_list'
+                  % name, 'w') as f:
             f.write(jsonpickle.dumps(path_list))
 
+
 def handle_output_parameter(name, value, sign, slices=None, data_root="/tmp"):
     if slices is not None:
         if isinstance(slices, list):
             assert isinstance(value, list) and len(slices) == len(value)
-            res = [{"dflow_list_item": v, "order": s} for v, s in zip(value, slices)]
+            res = [{"dflow_list_item": v, "order": s}
+                   for v, s in zip(value, slices)]
         else:
             res = [{"dflow_list_item": value, "order": slices}]
         with open(data_root + '/outputs/parameters/' + name, 'w') as f:
@@ -115,37 +144,52 @@ def handle_output_parameter(name, value, sign, slices=None, data_root="/tmp"):
             content["value"] = jsonpickle.dumps(value)
         with open(data_root + "/outputs/parameters/" + name, "w") as f:
             f.write(jsonpickle.dumps(content))
-    elif sign == str:
-        with open(data_root + '/outputs/parameters/' + name, 'w') as f:
-            f.write(value)
     else:
-        with open(data_root + '/outputs/parameters/' + name, 'w') as f:
-            f.write(jsonpickle.dumps(value))
+        if isinstance(sign, Parameter):
+            sign = sign.type
+        if sign == str:
+            with open(data_root + '/outputs/parameters/' + name, 'w') as f:
+                f.write(value)
+        else:
+            with open(data_root + '/outputs/parameters/' + name, 'w') as f:
+                f.write(jsonpickle.dumps(value))
+
 
 def copy_results_and_return_path_item(path, name, order, data_root="/tmp"):
     if path and os.path.exists(str(path)):
-        return {"dflow_list_item": copy_results(path, name, data_root), "order": order}
+        return {"dflow_list_item": copy_results(path, name, data_root),
+                "order": order}
     else:
         return {"dflow_list_item": None, "order": order}
 
+
 def copy_results(source, name, data_root="/tmp"):
     source = str(source)
-    if source.find(data_root + "/inputs/artifacts/") == 0: # if refer to input artifact
-        rel_path = source[source.find("/", len(data_root + "/inputs/artifacts/"))+1:] # retain original directory structure
+    # if refer to input artifact
+    if source.find(data_root + "/inputs/artifacts/") == 0:
+        # retain original directory structure
+        rel_path = source[source.find(
+            "/", len(data_root + "/inputs/artifacts/"))+1:]
         target = data_root + "/outputs/artifacts/%s/%s" % (name, rel_path)
         copy_file(source, target, shutil.copy)
+        if rel_path[:1] == "/":
+            rel_path = rel_path[1:]
         return rel_path
     else:
         target = data_root + "/outputs/artifacts/%s/%s" % (name, source)
         try:
             copy_file(source, target, os.link)
-        except:
+        except Exception:
             copy_file(source, target, shutil.copy)
+        if source[:1] == "/":
+            source = source[1:]
         return source
 
+
 def handle_empty_dir(path):
-    # touch an empty file in each empty dir, as object storage will ignore empty dirs
+    # touch an empty file in each empty dir, as object storage will ignore
+    # empty dirs
     for dn, ds, fs in os.walk(path, followlinks=True):
         if len(ds) == 0 and len(fs) == 0:
-            with open(os.path.join(dn, ".empty_dir"), "w") as f:
+            with open(os.path.join(dn, ".empty_dir"), "w"):
                 pass
